/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.config;

import edu.uiuc.ge.illigal.gale.distance.Distance;
import edu.uiuc.ge.illigal.gale.agents.Agent;

import java.io.*;
import java.util.*;

/**
 * This interface contains the access to the configuration object
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class FileGALEConfiguration 
extends GALEConfiguration 
implements Serializable
{
   //---------------------------------------------------------------------

   /** Loads a GA parameter configuration and features range from file. 
    *		
    *  @param sParamFile Contains the file name for the Ale parameters
    *                    configuration file
    *  @exception FileNotFoundException The requested GALE parameters
    *                                   configuration file is not found
    *  @exception IOException Some IO exception occurs when loading the GALE 
    *             parameters configuration file
    *  @exception Exception Class dynamic loading exception
    */
   public FileGALEConfiguration(String sParamFile)
   throws FileNotFoundException, IOException, Exception
   {
      String sLine = null;

      FileReader frIn = new FileReader(sParamFile);
      LineNumberReader lnrIn = new LineNumberReader (frIn);

      while ( null!=(sLine=lnrIn.readLine()) )
         processParamLine(sLine);

		lnrIn.close();
      frIn.close();
   }

   //---------------------------------------------------------------------

   /** Processes a line read from the configuration file.
	 *
    *  @param sLine Contains the file to be processed
    *  @exception IOException Some IO exception occurs when loading the GALE
    *                         parameters configuration file
    *  @exception Exception Class dynamic loading exception
    */

   private void processParamLine ( String sLine )
   throws IOException, Exception
   {
      int iParam = 0;
      String sCad = null;
      StringTokenizer stfl = new StringTokenizer ( sLine );

      if (sLine.startsWith("#") || sLine.equals("")) return;

      sCad = stfl.nextToken();
      if (sCad.equalsIgnoreCase("NumAttributes"))            iParam = 1;
      else if (sCad.equalsIgnoreCase("NumClasses"))          iParam = 2;
      else if (sCad.equalsIgnoreCase("Seed"))                iParam = 3;
      else if (sCad.equalsIgnoreCase("AgentInitAttributes")) iParam = 4;
      else if (sCad.equalsIgnoreCase("WorldIterations"))     iParam = 6;
      else if (sCad.equalsIgnoreCase("SomaticMutationP"))    iParam = 7;
      else if (sCad.equalsIgnoreCase("MergeP"))              iParam = 8;
      else if (sCad.equalsIgnoreCase("kThreshold"))          iParam = 9;
      else if (sCad.equalsIgnoreCase("MaxSplitP"))           iParam = 10;
      else if (sCad.equalsIgnoreCase("AgentMaxInitWidth"))   iParam = 11;
      else if (sCad.equalsIgnoreCase("AgentMaxInitDepth"))   iParam = 12;
      else if (sCad.equalsIgnoreCase("InitialAggregates"))   iParam = 14;
      else if (sCad.equalsIgnoreCase("MaxX"))                iParam = 15;
      else if (sCad.equalsIgnoreCase("MaxY"))                iParam = 16;
      else if (sCad.equalsIgnoreCase("MappingStrategy"))     iParam = 17;
      else if (sCad.equalsIgnoreCase("Prune"))               iParam = 18;
      else if (sCad.equalsIgnoreCase("TestMode"))            iParam = 19;
      else if (sCad.equalsIgnoreCase("Distance"))            iParam = 20;
      else if (sCad.equalsIgnoreCase("Agent"))               iParam = 21;
      else if (sCad.equalsIgnoreCase("SeedRuns"))            iParam = 22;
      else if (sCad.equalsIgnoreCase("InstancesProportion")) iParam = 23;
      else throw new IOException ("Wrong parameter at:"+sLine);

      if (!stfl.nextToken().equals("="))
        throw new IOException ("= expected at :"+sLine);

		Class cDis = null;
      sCad = stfl.nextToken();
      switch (iParam) {
         case  1: iNumFeatures       = Integer.parseInt(sCad);        break;
         case  2: iNumClasses        = Integer.parseInt(sCad);        break;
         case  3: lSeed              = Long.parseLong(sCad);
                  rndGen             = new Random(lSeed);             break;
         case  4: iAgentInitFeatures = Integer.parseInt(sCad);        break;
         case  6: iWorldIterations   = Integer.parseInt(sCad);        break;
         case  7: fSomaticMP         = Float.parseFloat(sCad); 		 break;
         case  8: fMergeP            = Float.parseFloat(sCad); 		 break;
         case  9: fKThreshold        = Float.parseFloat(sCad); 		 break;
         case 10: fMaxSplitP         = Float.parseFloat(sCad); 		 break;
         case 11: iAgentMaxInitWidth = Integer.parseInt(sCad);        break;
         case 12: iAgentMaxInitDepth = Integer.parseInt(sCad);        break;
         case 14: fInitialAggregates = Float.parseFloat(sCad);        break;
         case 15: iMaxX              = Integer.parseInt(sCad);        break;
         case 16: iMaxY              = Integer.parseInt(sCad);        break;
         case 17: sMappingStrategy   = sCad;                          break;
         case 18: if ( sCad.equalsIgnoreCase("true") )      bPrune = true;
                  else if ( sCad.equalsIgnoreCase("false")) bPrune = false;
						else throw new IOException(sCad+
			                                    " not a boolean value (Prune)");
						break;
         case 19: if ( sCad.equalsIgnoreCase("Best") )
							iTestMode = TEST_BEST;
                  else if ( sCad.equalsIgnoreCase("Bagging"))
							iTestMode = TEST_BAGGING;
						else throw new IOException(sCad+" not a valid test mode");
						break;
			case 20: cDis =
			            Class.forName("edu.uiuc.ge.illigal.gale.distance."+sCad);
						dis = (Distance)cDis.newInstance();
						break;
			case 21: cDis =
			            Class.forName("edu.uiuc.ge.illigal.gale.agents."+sCad+"Agent");
						aAgent = (Agent)cDis.newInstance();
						break;
         case 22: lSeedRuns          = Long.parseLong(sCad); 	break;
         case 23: fProportion        = Float.parseFloat(sCad); 		 break;
         }
   }

   //---------------------------------------------------------------------

}

